#include <stdbool.h>

#include "devices.h"
#include "lib.h"
#include "network.h"

/* ARP is defined in RFC 826. */

/* ARP commands */
#define REQUEST	0x0100
#define REPLY	0x0200

struct arp_header {
	unsigned short hrd;	/* format of hardware addr. should be 1 = eth */
	unsigned short pro;	/* format of protocol addr. should be 0x800 = ip */
	unsigned char hln;	/* length of hardware addr. should be 6 */
	unsigned char pln;	/* length of protocol addr. should be 4 */
	unsigned short op;	/* opcode (command) */
	/* the following is only valid for IP over Ethernet */
	unsigned char sha[6];	/* sender hardware address */
	unsigned char sip[4];	/* sender ip address */
	unsigned char tha[6];	/* target hardware address */
	unsigned char tip[4];	/* target ip address */
} __attribute__((packed));

/* handle ARP request */
static void arp_req(const struct arp_header *request)
{
	struct arp_header reply;

	/* check whether the ARP request was send to us */
	if (memcmp(ip_addr, request->tip, 4))
		return;

	/* answer the request */
	reply.hrd = 0x0100;
	reply.pro = 0x0008;
	reply.hln = 0x06;
	reply.pln = 0x04;
	reply.op = REPLY;
	memcpy(reply.sha, mac_addr, 6);
	memcpy(reply.sip, ip_addr, 4);
	memcpy(reply.tha, request->sha, 6);
	memcpy(reply.tip, request->sip, 4);

	/* send the reply */
	enc28j60_transmit(request->sha, ETH_ARP, 1, &reply,
			  sizeof(struct arp_header));
}

void arp_handle(void *pkg, const unsigned char *src __attribute__ ((unused)))
{
	const struct arp_header *arp_header = pkg;

	switch (arp_header->op) {
	case REQUEST:
		arp_req(arp_header);
		break;
	case REPLY:	/* not needed till now */
	default:
		break;
	}
}
