#ifndef DEVICES_H
#define DEVICES_H

/* our PLL multiplicator */
#define PLL_MUL		2

/* crystal clock in Hz */
#define CRYSTAL_CLOCK	14745600

/* CPU clock in Hz */
#define CPU_CLOCK	(CRYSTAL_CLOCK * PLL_MUL)

/* MAM cycles
 * According to the data sheet, MAM timings may be set:
 *  System Clock [MHz]	MAM
 *    0 - 20		1
 *   20 - 40		2
 *   40 - 60		3
 *  so:
 *   60 - 80		4
 *   80 - 100		5
 *  100 - 120		6
 *  120 - 140		7
 *  So, to get the optimal MAM timing for the system clock, we use: */
#define MAM_CCLKS	(CPU_CLOCK / 20000000 + 1)

#ifndef __ASSEMBLER__

/* GPIO registers */
#define GPIO_BASE	0xe0028000
#define IOPIN		(*(volatile unsigned int *) (GPIO_BASE + 0x0))
#define IOSET		(*(volatile unsigned int *) (GPIO_BASE + 0x4))
#define IODIR		(*(volatile unsigned int *) (GPIO_BASE + 0x8))
#define IOCLR		(*(volatile unsigned int *) (GPIO_BASE + 0xc))

/* in enc28j60.c: */

extern void enc28j60_init();

/* the frag argument specifies the number of fragments of the package. For every
 * fragment, you should specify the pointer to the beginning of the fragment and
 * the length of the fragment.
 * This way, it is possible to build up packets without needing much stack
 * memory and without needing much memcpy() calls. */
extern void enc28j60_transmit(const unsigned char *dest_macaddr,
			      unsigned short type, int frag, ...);

extern void enc28j60_recvloop() __attribute__ ((noreturn));

/* returns the number of bytes which could be received currently */
extern unsigned short enc28j60_get_recv_window();

/* My MAC address. First byte is 0x00 and the following are from /dev/random.
 * This should be unique in the network area up to the next gateway. */
#define MAC_ADDR	0x00,0x09,0x11,0xba,0xf7,0xad
extern const unsigned char mac_addr[6];

/* in mmc.c */

extern void mmc_init();
extern void mmc_read(unsigned int addr, unsigned int count, void *buf);

#endif	/* !__ASSEMBLER__ */

#endif	/* DEVICES_H */
