#ifndef FS_H
#define FS_H

#define S_IFMT		0170000
#define S_IFDIR		0040000
#define S_IFCHR		0020000
#define S_IFBLK		0060000
#define S_IFREG		0100000
#define S_IFIFO		0010000
#define S_IFLNK		0120000
#define S_IFSOCK	0140000

#define S_ISLNK(m)	(((m) & S_IFMT) == S_IFLNK)
#define S_ISREG(m)	(((m) & S_IFMT) == S_IFREG)
#define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
#define S_ISCHR(m)	(((m) & S_IFMT) == S_IFCHR)
#define S_ISBLK(m)	(((m) & S_IFMT) == S_IFBLK)
#define S_ISFIFO(m)	(((m) & S_IFMT) == S_IFIFO)
#define S_ISSOCK(m)	(((m) & S_IFMT) == S_IFSOCK)

struct inode {
	unsigned short mode;		/* file type, protection, etc.	*/
	unsigned short uid;		/* user id of the file's owner	*/
	unsigned long size;		/* current file size in bytes	*/
	unsigned long mtime;		/* modification time		*/
	unsigned char gid;		/* group id of the file's owner	*/
	unsigned char nlinks;		/* number of links to this file	*/
	unsigned short zone[9];		/* zone numbers			*/
};

struct dent {
	unsigned short ino;		/* inode number of the dirent	*/
	char name[30];			/* filename			*/
};

extern void ibmpart_init();

extern void minix_mount(unsigned int off);

/* something like pread() */
extern int minix_read_data(const struct inode *ino, unsigned int addr,
			   void *buf, int count);
/* something like open() */
extern int minix_open(const char *file);
/* something like fstat() */
extern void minix_get_inode(int nr, struct inode *ino);

/* is implementation-defined */
#define PATH_MAX	96

#endif
