#include "devices.h"
#include "fs.h"
#include "lib.h"

/* format of partition entry */
struct part {
	unsigned char bootind;		/* bit 7 indicates whether bootable */
	unsigned char start_chs[3];	/* start cylinder/head/sector */
	unsigned char sysind;		/* type of partition */
	unsigned char end_chs[3];	/* end cylinder/head/sector */
	unsigned int start;		/* number of first sector */
	unsigned int size;		/* partition size in sectors */
} __attribute__ ((packed));

/* format of partition table */
struct pt {
	struct part part[4];
	unsigned short magic;
} __attribute__ ((packed));

/* address of partition table */
#define PT_ADDR		0x1be

/* magic */
#define PT_MAGIC	0xaa55

/* sysind for minix filesystem */
#define MINIXFS		0x81

void ibmpart_init()
{
	struct pt pt;
	int i;

	mmc_read(PT_ADDR, sizeof(struct pt), &pt);

	if (pt.magic != PT_MAGIC)
		panic("No partition table found.");

	for (i = 0; i < 4; i++)
		if (pt.part[i].sysind == MINIXFS) {
			printf("Using partition #%i (%i MB) as fs.\n", i + 1,
			       pt.part[i].size >> 11);
			minix_mount(pt.part[i].start << 9);
			return;
		}

	panic("No minix fs partition found.");
}
