#include "lib.h"
#include "network.h"

/* ICMP is defined in RFC 792. */

/* ICMP commands */
#define ECHOREPLY	0
#define ECHOREQUEST	8

struct icmp_header {
	unsigned char command;
	unsigned char code;
	unsigned short checksum;
	unsigned short id;
	unsigned short sequence;
} __attribute__ ((packed));

static void icmp_reply(const struct icmp_header *req_hdr, const void *data,
		       unsigned short len, const unsigned char *srcmac,
		       const unsigned char *srcip)
{
	/* thats an odd (but in this case needed, due to the dynamic size) way
	 * to implement an union: */
	struct icmp_header *rpl_hdr;
	char rpl_c[len];
	rpl_hdr = (void *) rpl_c;

	/* build header */
	rpl_hdr->command = ECHOREPLY;
	rpl_hdr->code = 0;
	rpl_hdr->checksum = 0;	/* calculated later */
	rpl_hdr->id = req_hdr->id;
	rpl_hdr->sequence = req_hdr->sequence;

	/* copy data */
	memcpy(&rpl_c[sizeof(struct icmp_header)], data,
	       len - sizeof(struct icmp_header));

	/* calculate checksum */
	rpl_hdr->checksum = create_checksum(rpl_c, len);

	/* transmit the reply */
	ip_transmit(srcmac, srcip, IP_ICMP, rpl_c, len);
}

void icmp_handle(void *pkg, unsigned short len,
		 const unsigned char *srcmac, const unsigned char *srcip)
{
	const struct icmp_header *icmp_header = pkg;

	if (icmp_header->command != ECHOREQUEST)
		/* unsupported */
		return;

	if (1500 < len || len < sizeof(struct icmp_header))
		/* bad, but often tried trick: packet is too short for a real
		 * ICMP packet, or too big for fitting in one ethernet packet */
		return;

	pkg += sizeof(struct icmp_header);

	icmp_reply(icmp_header, pkg, len, srcmac, srcip);
}
