#include "devices.h"
#include "lib.h"
#include "network.h"

/* IP is defined by RFC 791. */

struct ip_header {
	unsigned char hv;	/* lower bits: header length in words
				 * upper bits: 4 */
	unsigned char tos;
	unsigned short length;
	unsigned short id;
	unsigned short frag;	/* 0x0040 = don't fragment */
	unsigned char ttl;
	unsigned char protocol;
	unsigned short checksum;
	unsigned char src[4];
	unsigned char dst[4];
} __attribute__((packed));

const unsigned char ip_addr[4] = { IP_ADDR };

static const struct {
	unsigned char type;
	void (*handler)(void *pkg, unsigned short len,
			const unsigned char *srcmac,
			const unsigned char *srcip);
} ip_types[] = {
	{ IP_ICMP, icmp_handle },
	{ IP_TCP,  tcp_handle  }
};

void ip_handle(void *pkg, const unsigned char *src)
{
	unsigned int i;
	const struct ip_header *ip_header = pkg;
	unsigned short hdrlen, len;

	hdrlen = (ip_header->hv & 0xf) * 4;

	/* header is in ip_header. skip it for the pkg pointer */
	pkg += hdrlen;

	len = ntohs(ip_header->length) - hdrlen;

	/* forward it to the handler */
	for (i = 0; i < ARRAY_SIZE(ip_types); i++)
		if (ip_types[i].type == ip_header->protocol)
			ip_types[i].handler(pkg, len, src, ip_header->src);
}

void ip_transmit(const unsigned char *dstmac, const unsigned char *dstip,
		 unsigned short type, const void *pkg, unsigned int len)
{
	struct ip_header ip_header;

	/* build up header */
	ip_header.hv = 0x40 | (sizeof(struct ip_header) / 4);
	ip_header.tos = 0;
	ip_header.length = htons(sizeof(struct ip_header) + len);
	ip_header.id = 0;
	ip_header.frag = 0x40;
	ip_header.ttl = TTL;
	ip_header.protocol = type;
	ip_header.checksum = 0;	/* filled in later */
	memcpy(ip_header.src, ip_addr, 4);
	memcpy(ip_header.dst, dstip, 4);

	/* create checksum */
	ip_header.checksum = create_checksum(&ip_header,
					     sizeof(struct ip_header));

	/* send packet */
	enc28j60_transmit(dstmac, ETH_IP, 2, &ip_header, sizeof(ip_header), pkg,
			  len);
}
