#ifndef LIB_H
#define LIB_H

#include <stdarg.h>

/* limits */
#define INT_MIN	(-INT_MAX - 1)
#define INT_MAX	2147483647

/* stdio */

extern int vsprintf(char *, const char *, va_list);
extern int sprintf(char *, const char *, ...);

extern int printf(const char *, ...);
extern int puts(const char *);

/* string */

extern void *memcpy(void *, const void *, int);
extern int memcmp(const void *s1, const void *s2, int n);
extern char *strchr(const char *, int);
extern int strlen(const char *);
extern int strncmp(const char *s1, const char *s2, int n);
extern int strcmp(const char *s1, const char *s2);
extern char *strdup(const char *s);
extern char *strcpy(char *d, const char *s);

/* netinet/in */

extern unsigned short htons(unsigned short);
extern unsigned short ntohs(unsigned short);
extern unsigned int htonl(unsigned int);
extern unsigned int ntohl(unsigned int);

/* stdlib */

extern void *malloc(int size);
extern void free(const void *ptr);

/* time */

struct tm {
	char sec;	/* 0..60	seconds since start of minute	*/
	char min;	/* 0..59	minutes since start of hour	*/
	char hour;	/* 0..23	hours since start of day	*/
	char mday;	/* 1..31	days since start of month + 1	*/
	char mon;	/* 0..11	month since start of year	*/
	char year;	/* -inf..inf	years since 1900		*/
};

extern void gmtime(int tl, struct tm *ts);

/* nowhere */

#define ARRAY_SIZE(x)	(sizeof(x) / sizeof(x[0]))

#define min(a, b)	((a < b) ? a : b)
#define max(a, b)	((a > b) ? a : b)

/* calculates e.g. IP checksums */
extern unsigned short create_checksum(const void *buf, int size);

/* halts the system.
 *
 * Use it for fatal error conditions, which are either more-or-less volatile
 * (e.g. if the MMC was removed from the slot), or depending on (non-static)
 * system/peripherial configuration (e.g. no partition table on MMC).
 *
 * Do NOT use it for really static error conditions (e.g. if there is no
 * enc28j60 on board). Using it for such cases would be quite a bit stupid. */
extern void panic(const char *reason) __attribute__ ((noreturn));

/* returns the number of needed blocks with a size of 'blocksize' to house
 * 'size' bytes.
 *
 * blocks(4000, 4096) = 1, blocks(4096, 4096) = 1, blocks(4097, 4096) = 2 */
extern unsigned int blocks(int size, int blocksize);

#define TOSTR(x...)	#x
#define STR(y)		TOSTR(y)

#endif
