#include "devices.h"

.text
.arm

.global _start
_start:
	ldr	pc, reset_ptr
	ldr	pc, undefined_ptr
	ldr	pc, swi_ptr
	ldr	pc, prog_abort_ptr
	ldr	pc, data_abort_ptr
	nop
	ldr     pc, irq_ptr
	ldr	pc, [pc, #-0x0ff0]
	ldr	pc, fiq_ptr

reset_ptr:	.word	bootstrap
undefined_ptr:	.word	unexpected_exception
swi_ptr:	.word	unexpected_exception
prog_abort_ptr:	.word	unexpected_exception
data_abort_ptr:	.word	unexpected_exception
reserved_ptr:	.word	0
irq_ptr:	.word	unexpected_exception
fiq_ptr:	.word	unexpected_exception

unexpected_exception:
	b	unexpected_exception

bootstrap:
	/* setup phase locked loop */
	ldr	r0,	=0xe01fc080	/* base address */
	mov	r1,	#(PLL_MUL - 1)	/* set multiplier */
	str	r1,	[r0, #0x4]
	mov	r1,	#1		/* enable PLL */
	str	r1,	[r0, #0x0]
	mov	r2,	#0xaa		/* send feed seq to PLL */
	mov	r3,	#0x55
	str	r2,	[r0, #0xc]
	str	r3,	[r0, #0xc]
1:	ldr	r1,	[r0, #0x8]	/* wait for pll till it is locked */
	ands	r1, r1, #(1 << 10)
	beq	1b
	mov	r1,	#3		/* enable and connect PLL clock */
	str	r1,	[r0, #0x0]
	str	r2,	[r0, #0xc]	/* send feed again */
	str	r3,	[r0, #0xc]

	/* setup memory accelerator module */
	ldr	r0,	=0xe01fc000	/* base address */
	mov	r1,	#(MAM_CCLKS)
	str	r1,	[r0, #0x4]
	mov	r1,	#2		/* fully enable MAM */
	str	r1,	[r0, #0x0]

	/* setup stack pointer */
	ldr	sp,	=0x40010000

	/* set vpbdiv */
	ldr	r0,	=0xe01fc100
	mov	r1,	#1
	str	r1,	[r0, #0x0]

	/* configure pin 0.0 and 0.1 as UART pins, and pin 0.4-0.7 as SPI
	 * pins */
	ldr	r0,	=0xe002c000
	ldr	r1,	=0x00005505
	str	r1,	[r0, #0x0]

	/* NOTE: when needed, copy data */

	/* gcc expects .bss to be zeroed. so do that now */
	mov	r0,	#0
	ldr	r1, 	=0x40000000
	ldr	r2,	=_bss_end__
2:	cmp	r1,	r2
	strlo	r0,	[r1],	#4
	blo	2b

	bl	main
3:	b	3b	/* in case main returns, go into an endless loop */
